//=============================================================================
// file: ImpressionistView.h
// desc:
//		 Implements the view and controller parts of the standard MVC (model-
//		 view-controller) pattern.  Normally these would be in separate classes
//		 but this particular application is too simple for that to be
//		 practical.  The ImpressionistView class handles all drawing and user
//		 interaction to and from the viewports.
//=============================================================================

#ifndef __IMPRESSIONIST_VIEW_HEADER__
#define __IMPRESSIONIST_VIEW_HEADER__

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_file_chooser.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Choice.H>

#include "ImpressionistModel.h"
#include "OriginalView.h"
#include "PaintView.h"
#include "AbstractBrush.h"

//==========[ class ImpressionistView ]========================================

class ImpressionistView {

private:

	//---[ Model Pointer ]---------------------------------

	ImpressionistModel*	mModel;

	//---[ Brushes ]---------------------------------------

	AbstractBrush**		mBrushes;
	
	//---[ FLTK Widgets ]----------------------------------

	Fl_Window*			mMainWindow;
	Fl_Menu_Bar*		mMenubar;
								
	Fl_Window*			mBrushDialog;
	Fl_Slider*			mBrushSizeSlider;
	Fl_Choice*			mBrushTypeChoice;

	//---[ OpenGL Viewports ]------------------------------

		// viewport for the Impressionist rendering
	PaintView*			mPaintView;
		// viewport for the original, unaltered image
	OriginalView*		mOrigView;

	//---[ FLTK Callbacks ]--------------------------------

		// called when File->Load Image is selected
	static void loadImageCB( Fl_Menu_* o, void* v );
		// called when File->Save Image is selected
	static void saveImageCB( Fl_Menu_* o, void* v );
		// called when File->Brushes is selected
	static void brushesCB( Fl_Menu_* o, void* v );
		// called when File->Exit is selected
	static void exitCB( Fl_Menu_* o, void* v );
		// called when Help->About is selected
	static void aboutCB( Fl_Menu_* o, void* v );
		// called when the main window is closed
	static void windowCloseCB( Fl_Widget* o, void* v );

		// called when the size slider is used
	static void sizeSlidesCB( Fl_Widget* o, void* v );
		// called when the brush choice menu is used
	static void brushChoiceCB( Fl_Widget* o, void* v );

	
	//---[ Static Variables ]------------------------------

		// array of menu items for the main menu bar
	static Fl_Menu_Item menuItems[];
		// array of menu items for the brush type pop-up
	static Fl_Menu_Item brushTypeMenu[];

public:

	//---[ Constructor ]-----------------------------------

	ImpressionistView( ImpressionistModel* model );

	//---[ Visibility Methods ]----------------------------

	void show();
	void hide();
};

#endif
