#include "ImpressionistModel.h"
#include "OriginalView.h"

//==========[ Macro Definitions ]==============================================

#ifndef WIN32
#define min(a, b)	( ( (a)<(b) ) ? (a) : (b) )
#endif

//==========[ OriginalView Methods ]===========================================

//----------[ Constructor ]----------------------------------------------------

OriginalView::OriginalView( ImpressionistModel* model, int x, int y, int w, int h, const char *l )
		: Fl_Gl_Window(x,y,w,h,l) {
	mModel = model;
}

//----------[ draw ]-----------------------------------------------------------

void OriginalView::draw() {

	// this should only be executed the first time draw is called
	if( !valid() ) {
		glClearColor(0.7f, 0.7f, 0.7f, 1.0);

		// we are using OpenGL as a 2D API, so turn off depth testing
		glDisable( GL_DEPTH_TEST );

		// we need to read from the front buffer to capture paint strokes
		glReadBuffer( GL_FRONT );

		// set the camera view to a simple orthographic projection
		ortho();
	}

	// clear the color information
	glClear( GL_COLOR_BUFFER_BIT );

	// if there is an image loaded, draw it
	if( mModel->imageLoaded() ) {

		// the pixel storage structure is RGB triples in a packed,
		// row-major array going from left to right and bottom to top.

		int bitmapWidth = mModel->getBitmapWidth();
		int bitmapHeight = mModel->getBitmapHeight();

		int	windowWidth = w();
		int windowHeight = h();

		int drawWidth = min( windowWidth, bitmapWidth );
		int drawHeight = min( windowHeight, bitmapHeight );

		// figure out the correct starting row and pixel
		int startrow = bitmapHeight - drawHeight;
		GLvoid* bitstart = mModel->getOriginalBitmap() + 3 *
				   ((bitmapWidth * startrow));

		// render the bitmap information to the OpenGL surface
		glRasterPos2i( 0, windowHeight - drawHeight );
		glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
		glPixelStorei( GL_UNPACK_ROW_LENGTH, bitmapWidth );
		glDrawBuffer( GL_BACK );
		glDrawPixels( drawWidth, drawHeight, GL_RGB,
					  GL_UNSIGNED_BYTE, bitstart );
	}

	glFlush();
}

//----------[ refresh ]--------------------------------------------------------

void OriginalView::refresh() {
	redraw();
}

//----------[ resizeWindow ]---------------------------------------------------

void OriginalView::resizeWindow( int width, int height ) {
	resize( x(), y(), width, height );
}