//=============================================================================
// file: Impressionist.h
// desc:
//		 Standard include file for the Impressionist project.  Defines the
//		 utility classes "Color" and "Point" as well as including the standard
//		 FLTK and OpenGL headers.
//=============================================================================

#ifndef IMPRESSIONIST_H
#define IMPRESSIONIST_H

#include <stdio.h>

#include <FL/Fl.h>
#include <FL/gl.h>
#include <GL/glu.h>

//==========[ class Color ]====================================================
// desc: Simple structure that defines an RGB triple using unsigned characters.
//		 To use this with OpenGL use the glColor3ub method.
//=============================================================================

class Color {

public:

	//---[ Constructors ]----------------------------------

	Color( unsigned char rr, unsigned char gg, unsigned char bb ) { r = rr; g = gg; b = bb; }
	Color( const Color& c ) { r = c.r; g = c.g; b = c.b; }

	//---[ Public Variables ]------------------------------

	unsigned char r, g, b;
};

//==========[ class Point ]====================================================
// desc: Defines a simple two-dimensional point in the integers.
//=============================================================================

class Point {

public:

	//---[ Constructors ]----------------------------------

	Point() { x = 0; y = 0; };
	Point(int xx, int yy) { x = xx; y = yy; };

	//---[ Public Variables ]------------------------------

	int x, y;
};

#endif